@@ -569,7 +569,8 @@ class gaussian_kde(object):
 
         self.covariance = self._data_covariance * self.factor**2
         self.inv_cov = self._data_inv_cov / self.factor**2
-        self._norm_factor = sqrt(linalg.det(2*pi*self.covariance))
+        L = linalg.cholesky(self.covariance*2*pi)
+        self.log_det = 2*np.log(np.diag(L)).sum()
 
     def pdf(self, x):
         """
@@ -603,22 +604,22 @@ class gaussian_kde(object):
 
         if m >= self.n:
             # there are more points than data, so loop over data
-            energy = zeros((self.n, m), dtype=float)
+            energy = np.empty((self.n, m), dtype=float)
             for i in range(self.n):
                 diff = self.dataset[:, i, newaxis] - points
                 tdiff = dot(self.inv_cov, diff)
-                energy[i] = sum(diff*tdiff, axis=0) / 2.0
-            result = logsumexp(-energy.T,
-                               b=self.weights / self._norm_factor, axis=1)
+                energy[i] = sum(diff*tdiff, axis=0)
+            log_to_sum = 2.0 * np.log(self.weights) - self.log_det - energy.T
+            result = logsumexp(0.5 * log_to_sum, axis=1)
         else:
             # loop over points
-            result = zeros((m,), dtype=float)
+            result = np.empty((m,), dtype=float)
             for i in range(m):
                 diff = self.dataset - points[:, i, newaxis]
                 tdiff = dot(self.inv_cov, diff)
-                energy = sum(diff * tdiff, axis=0) / 2.0
-                result[i] = logsumexp(-energy, b=self.weights /
-                                      self._norm_factor)
+                energy = sum(diff * tdiff, axis=0)
+                log_to_sum = 2.0 * np.log(self.weights) - self.log_det - energy
+                result[i] = logsumexp(0.5 * log_to_sum)
 
         return result
 
